c----------------------------------------------------------------------- 
c 
c     This file is part of the Test Set for IVP solvers 
c     http://www.dm.uniba.it/~testset/ 
c 
c        (ODE only) DOPRI5 (NONSTIFF solver) driver 
c 
c     DISCLAIMER: see 
c     http://www.dm.uniba.it/~testset/disclaimer.php 
c 
c     The most recent version of this source file can be found at 
c     http://www.dm.uniba.it/~testset/src/drivers/dopri5d.f 
c 
c     This is revision 
c     $Id: dopri5d.f,v 1.7 2006/10/02 10:19:08 testset Exp $ 
c 
c----------------------------------------------------------------------- 
      program doprid 
 
      integer mxndgl, nrdens, lwork, liwork 
      parameter (mxndgl=400,nrdens=2) 
      parameter (lwork=8*mxndgl+5*nrdens+20,liwork=nrdens+20) 
 
      integer n, ndisc, mljac, mujac, mlmas, mumas, ind(mxndgl), 
     +        iout, itol, iwork(liwork), ipar(2), idid 
      logical numjac,nummas, consis, tolvec 
      double precision y(mxndgl), yprime(mxndgl), t(0:100), 
     +                 h0, rtol(mxndgl), atol(mxndgl), 
     +                 work(lwork), rpar(1) 
 
      double precision yref(mxndgl) 
      external odefcn, solout 
      character fullnm*40, problm*8, type*3 
      character driver*8, solver*8 
      parameter (driver = 'dopri5d', solver='DOPRI5') 
 
      double precision solmax 
      real gettim, timtim, cputim 
      external gettim 
      double precision scd, mescd 
 
 
      character  fileout*140,  filepath*100 
      character formatout*30,  namefile*100 
      logical printsolout, solref, printout 
      integer nindsol, indsol(mxndgl) 
      integer i, j, icount(17:20) 
 
      do 10 i=1,20 
         iwork(i) = 0 
         work(i)  = 0d0 
   10 continue 
      iout = 0 
 
c----------------------------------------------------------------------- 
c     check the problem definition interface date 
c----------------------------------------------------------------------- 
 
      call chkdat(driver,solver,20060828) 
 
c----------------------------------------------------------------------- 
c     get the problem dependent parameters 
c----------------------------------------------------------------------- 
 
      call prob(fullnm,problm,type, 
     +          n,ndisc,t, 
     +          numjac,mljac,mujac, 
     +          nummas,mlmas,mumas, 
     +          ind) 
      if (type.eq.'IDE') then 
         print *, 'DOPRI5D: ERROR: ', 
     +            'DOPRI5 can not solve IDEs' 
         stop 
      elseif (type.eq.'DAE') then 
         print *, 'DOPRI5D: ERROR: ', 
     +            'DOPRI5 can not solve DAEs' 
         stop 
      elseif (type.ne.'ODE') then 
         print *, 'DOPRI5D: ERROR: ', 
     +            'unknown Test Set problem type', type 
         stop 
      endif 
 
c----------------------------------------------------------------------- 
c     get the initial values 
c----------------------------------------------------------------------- 
 
      call init(n,t(0),y,yprime,consis) 
 
c----------------------------------------------------------------------- 
c     read the tolerances 
c----------------------------------------------------------------------- 
 
      call getinp(driver,problm,solver,fullnm, 
     +            tolvec,rtol,atol,h0,solmax) 
      
      call settolerances(n,rtol,atol,tolvec) 
 
      if (tolvec) then 
         itol = 1 
      else 
         itol = 0 
      endif 
 
      call  setoutput(n,solref,printsolout, 
     +                     nindsol,indsol) 
 
      if (printsolout) then 
          print *, 'DOPRI5D: WARNING: ', 
     +             'this option is not yet implemented;', 
     +             'we set printsolout  = .false. ' 
          printsolout = .false. 
      end if 
 
c----------------------------------------------------------------------- 
c     call of the subroutine DOPRI5 
c----------------------------------------------------------------------- 
 
      do 20 i=17,20 
         icount(i) = 0 
   20 continue 
 
      timtim = gettim() 
      timtim = gettim() - timtim 
 
      cputim = gettim() 
 
      do 40 i=0,ndisc 
 
         call dopri5(n,odefcn,t(i),y,t(i+1), 
     +               rtol,atol,itol, 
     +               solout,iout, 
     +               work,lwork,iwork,liwork,rpar,ipar,idid) 
 
         do 30 j=17,20 
            icount(j) = icount(j) + iwork(j) 
   30    continue 
 
         if (idid.ne.1) then 
            print *, 'DOPRI5D: ERROR: ', 
     +               'DOPRI5 returned IDID = ', idid 
            stop 
         endif 
 
   40 continue 
 
      cputim = gettim() - cputim - timtim 
 
      do 50 i=17,20 
         iwork(i) = icount(i) 
   50 continue 
 
c----------------------------------------------------------------------- 
c     print numerical solution in endpoint and integration statistics 
c----------------------------------------------------------------------- 
 
      printout = .true. 
      if (solref) then  
         call solut(n,t(ndisc+1),yref) 
         call getscd(mescd,scd,n,yref,y,problm,tolvec,atol,rtol, 
     +            printout) 
      else 
        call printsol(n,y,problm) 
      end if 
          
      call report( 
     +   driver,problm,solver, 
     +   rtol,atol,h0,solmax, 
     +   iwork,cputim,scd,mescd 
     +) 
 
      end 
 
      subroutine solout(nr,xold,x,y,n,con,icomp,nd,rpar,ipar,irtrn) 
      integer nr,n,nd,icomp(nd),ipar(*),irtrn 
      double precision xold,x,y(n),con(5*nd),rpar(*) 
      return 
      end 
 
C======================================================================= 
C     `Test Set for IVP Solvers' ODE wrapper for DOPRI5 
C======================================================================= 
c 
c     since in DOPRI5 the format of the subroutine containing the 
c     derivative function differs from the format in the 
c     testset, we transform it 
c 
c----------------------------------------------------------------------- 
      subroutine odefcn(n,x,y,f,rpar,ipar) 
      integer n,ipar(*) 
      double precision x,y(n),f(n),rpar(*) 
      integer ierr 
      ierr = 0 
      call feval(n,x,y,y,f,ierr,rpar,ipar) 
      if (ierr.ne.0) then 
         print *, 'DOPRI5D: ERROR: ', 
     +            'DOPRI5 can not handle FEVAL IERR' 
         stop 
      endif 
      return 
      end 
